/*
	File:			VLc.h

	Function:		Master header for a version of the VL library based on complex numbers.
					The various classes are named Vecc, Mat3c, SparseVecc, etc.
					
	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott
 */


#ifndef __VLf__
#define __VLf__

#ifdef __VL__
#include "VLUndef.h"
#endif

#include "Basics.h"
#include <complex.h>

inline Real len(float_complex c)
{ return(sqrt(sqr(real(c)) + sqr(imag(c)))); }
inline Real sqrlen(float_complex c)
{ return(sqr(real(c)) + sqr(imag(c))); }
inline Void SetReal(float_complex &a, Real b)
{ a = b; }

#define VL_V_REAL float_complex
#define VL_V_SUFF(X) X ## c
#define VL_M_REAL float_complex
#define VL_M_SUFF(X) X ## c

#include "Vec2.h"
#include "Vec3.h"
#include "Vec4.h"
#include "Vec.h"
#include "SparseVec.h"

#include "Mat2.h"
#include "Mat3.h"
#include "Mat4.h"
#include "Mat.h"
#include "SparseMat.h"
#include "Solve.h"
#include "Transform.h"

#endif
